@@ -1,7 +1,8 @@
 """
 The imageutils module contains general utilities for applying to images.
 """
-
+# Maintain python 2 backwards compatibility
+from __future__ import print_function
 # import the C++ extension into this level
 from ._imageutils import *
 import rsgislib 
@@ -241,7 +242,7 @@ A function which returns the WKT string representing the projection of the input
     rasterDS = None
     return projStr
 
-def resampleImage2Match(inRefImg, inProcessImg, outImg, gdalformat, interpMethod, datatype=None):
+def resampleImage2Match(inRefImg, inProcessImg, outImg, gdalformat, interpMethod, datatype=None, noDataVal=None, multicore=False):
     """
 A utility function to resample an existing image to the projection and/or pixel size of another image.
 
@@ -253,31 +254,17 @@ Where:
 * gdalformat is the gdal format for the output image.
 * interpMethod is the interpolation method used to resample the image [bilinear, lanczos, cubicspline, nearestneighbour, cubic, average, mode]
 * datatype is the rsgislib datatype of the output image (if none then it will be the same as the input file).
+* multicore - use multiple processing cores (Default = False)
 
-"""   
-    dataset = gdal.Open(inProcessImg, gdal.GA_ReadOnly)
-    if dataset == None:
-        raise Exception("Could not open the inProcessImg.")
-    numBands = dataset.RasterCount
-    gdalDType = dataset.GetRasterBand(1).DataType
-    dataset = None
+""" 
+    rsgisUtils = rsgislib.RSGISPyUtils()
+    numBands = rsgisUtils.getImageBandCount(inProcessImg)
+    if noDataVal == None:
+        noDataVal = rsgisUtils.getImageNoDataValue(inProcessImg)
+    
     if datatype == None:
-        if gdalDType == gdal.GDT_Byte:
-            datatype = rsgislib.TYPE_8UINT
-        elif gdalDType == gdal.GDT_Int16:
-            datatype = rsgislib.TYPE_16INT
-        elif gdalDType == gdal.GDT_Int32:
-            datatype = rsgislib.TYPE_32INT
-        elif gdalDType == gdal.GDT_UInt16:
-            datatype = rsgislib.TYPE_16UINT
-        elif gdalDType == gdal.GDT_UInt32:
-            datatype = rsgislib.TYPE_32UINT         
-        elif gdalDType == gdal.GDT_Float32:
-            datatype = rsgislib.TYPE_32FLOAT
-        elif gdalDType == gdal.GDT_Float64:
-            datatype = rsgislib.TYPE_64FLOAT
-        else:
-            raise Exception("Data type of the input file was not recognised or known.")
+        datatype = rsgisUtils.getRSGISLibDataTypeFromImg(inProcessImg)
+        
     interpolationMethod = gdal.GRA_NearestNeighbour
     if interpMethod == 'bilinear':
         interpolationMethod = gdal.GRA_Bilinear 
@@ -296,12 +283,30 @@ Where:
     else:
         raise Exception("Interpolation method was not recognised or known.")
     
-    rsgislib.imageutils.createCopyImage(inRefImg, outImg, numBands, 0, gdalformat, datatype)
+    backVal = 0.0
+    haveNoData = False
+    if noDataVal != None:
+        backVal = float(noDataVal)
+        haveNoData = True
+    
+    rsgislib.imageutils.createCopyImage(inRefImg, outImg, numBands, backVal, gdalformat, datatype)
 
     inFile = gdal.Open(inProcessImg, gdal.GA_ReadOnly)
     outFile = gdal.Open(outImg, gdal.GA_Update)
- 
-    gdal.ReprojectImage(inFile, outFile, None, None, interpolationMethod, 0.0, 0.0, gdal.TermProgress)
+
+    wrpOpts = []
+    if multicore:
+        if haveNoData:
+            wrpOpts = gdal.WarpOptions(resampleAlg=interpolationMethod, srcNodata=noDataVal, dstNodata=noDataVal, multithread=True, callback=gdal.TermProgress)    
+        else:
+            wrpOpts = gdal.WarpOptions(resampleAlg=interpolationMethod, multithread=True, callback=gdal.TermProgress)
+    else:
+        if haveNoData:
+            wrpOpts = gdal.WarpOptions(resampleAlg=interpolationMethod, srcNodata=noDataVal, dstNodata=noDataVal, multithread=False, callback=gdal.TermProgress)    
+        else:
+            wrpOpts = gdal.WarpOptions(resampleAlg=interpolationMethod, multithread=False, callback=gdal.TermProgress)
+    
+    gdal.Warp(outFile, inFile, options=wrpOpts)
     
     inFile = None
     outFile = None
@@ -309,7 +314,7 @@ Where:
 
 
 
-def reprojectImage(inputImage, outputImage, outWKT, gdalformat='KEA', interp='cubic', inWKT=None, noData=0.0, outPxlRes='image', snap2Grid=True):
+def reprojectImage(inputImage, outputImage, outWKT, gdalformat='KEA', interp='cubic', inWKT=None, noData=0.0, outPxlRes='image', snap2Grid=True, multicore=False):
     """
 This function provides a tool which uses the gdalwarp function to reproject an input image.
 
@@ -327,6 +332,7 @@ Where:
             2) 'auto' where an output resolution maintaining the image size of the input image will be used
             3) provide a floating point value for the image resolution (note. pixels will be sqaure) 
 * snap2Grid is a boolean specifying whether the TL pixel should be snapped to a multiple of the pixel resolution (Default is True).
+* nCores - the number of processing cores available for processing (-1 is all cores: Default=-1)
     """    
     rsgisUtils = rsgislib.RSGISPyUtils()
     
@@ -479,8 +485,15 @@ Where:
         outImgDS.GetRasterBand(i+1).SetNoDataValue(noData)
     
     print("Performing the reprojection")
-    gdal.ReprojectImage(inImgDS, outImgDS, None, None, eResampleAlg, 0.0, 0.0, gdal.TermProgress )  
     
+    wrpOpts = []
+    if multicore:
+        wrpOpts = gdal.WarpOptions(resampleAlg=eResampleAlg, srcNodata=noData, dstNodata=noData, multithread=True, callback=gdal.TermProgress)
+    else:
+        wrpOpts = gdal.WarpOptions(resampleAlg=eResampleAlg, srcNodata=noData, dstNodata=noData, multithread=False, callback=gdal.TermProgress)    
+
+    gdal.Warp(outImgDS, inImgDS, options=wrpOpts)
+
     inImgDS = None
     outImgDS = None    
 
@@ -671,12 +684,14 @@ rsgislib.imageutils.mergeExtractedHDF5Data(inTrainSamples, cloudTrainSamples)
     fH5Out.close()
 
 
-def doImagesOverlap(image1, image2):
+def doImagesOverlap(image1, image2, overThres=0.0):
     """
 Function to test whether two images overlap with one another.
+If the images have a difference projection/coordinate system then corners 
 
 * image1 - path to first image
 * image2 - path to second image
+* overThres - the amount of overlap required to return true (e.g., at least 1 pixel)
 
 Returns:
 
@@ -693,6 +708,11 @@ print("Images Overlap: " + str(overlap))
 """
     overlap = True
     
+    projSame = False
+    rsgisUtils = rsgislib.RSGISPyUtils()
+    if rsgisUtils.doGDALLayersHaveSameProj(image1, image2):
+        projSame = True
+    
     img1DS = gdal.Open(image1, gdal.GA_ReadOnly)
     if img1DS is None:
         raise rsgislib.RSGISPyException('Could not open image: ' + image1)
@@ -718,12 +738,26 @@ print("Images Overlap: " + str(overlap))
     
     img1BRX = img1GeoTransform[0] + (img1DS.RasterXSize * img1GeoTransform[1])
     img1BRY = img1GeoTransform[3] + (img1DS.RasterYSize * img1GeoTransform[5])
-        
-    img2TLX = img2GeoTransform[0]
-    img2TLY = img2GeoTransform[3]
     
-    img2BRX = img2GeoTransform[0] + (img2DS.RasterXSize * img2GeoTransform[1])
-    img2BRY = img2GeoTransform[3] + (img2DS.RasterYSize * img2GeoTransform[5])
+    if projSame:
+        img2TLX = img2GeoTransform[0]
+        img2TLY = img2GeoTransform[3]
+        
+        img2BRX = img2GeoTransform[0] + (img2DS.RasterXSize * img2GeoTransform[1])
+        img2BRY = img2GeoTransform[3] + (img2DS.RasterYSize * img2GeoTransform[5])
+    else:
+        inProj = osr.SpatialReference()
+        inProj.ImportFromEPSG(int(rsgisUtils.getEPSGCode(image2)))
+        
+        outProj = osr.SpatialReference()
+        outProj.ImportFromEPSG(int(rsgisUtils.getEPSGCode(image1)))
+        
+        if int(rsgisUtils.getEPSGCode(image1)) == 4326:
+            img2TLY, img2TLX = rsgisUtils.reprojPoint(inProj, outProj, img2GeoTransform[0], img2GeoTransform[3])
+            img2BRY, img2BRX = rsgisUtils.reprojPoint(inProj, outProj, (img2GeoTransform[0] + (img2DS.RasterXSize * img2GeoTransform[1])), (img2GeoTransform[3] + (img2DS.RasterYSize * img2GeoTransform[5])))
+        else:
+            img2TLX, img2TLY = rsgisUtils.reprojPoint(inProj, outProj, img2GeoTransform[0], img2GeoTransform[3])
+            img2BRX, img2BRY = rsgisUtils.reprojPoint(inProj, outProj, (img2GeoTransform[0] + (img2DS.RasterXSize * img2GeoTransform[1])), (img2GeoTransform[3] + (img2DS.RasterYSize * img2GeoTransform[5]))) 
         
     xMin = img1TLX
     xMax = img1BRX
@@ -739,9 +773,9 @@ print("Images Overlap: " + str(overlap))
     if img2TLY < yMax:
         yMax = img2TLY
         
-    if xMax - xMin <= 0:
+    if xMax - xMin <= overThres:
         overlap = False
-    elif yMax - yMin <= 0:
+    elif yMax - yMin <= overThres:
         overlap = False
     
     return overlap
@@ -787,4 +821,49 @@ Where:
     applier.apply(_popPxlsRanVals, infiles, outfiles, otherargs, controls=aControls)
 
 
+def extractImgPxlSample(inputImg, pxlNSample, noData=None):
+    """
+A function which extracts a sample of pixels from the 
+input image file to a number array.
+
+* inputImg - the image from which the random sample will be taken.
+* pxlNSample - the sample to be taken (e.g., a value of 100 will sample every 100th, valid (if noData specified), pixel)
+* noData - provide a no data value which is to be ignored during processing. If None then ignored (Default: None)
+
+return:: outputs a numpy array (n sampled values, n bands)
+
+""" 
+    # Import the RIOS image reader
+    from rios.imagereader import ImageReader
+    
+    first = True
+    reader = ImageReader(inputImg, windowxsize=200, windowysize=200)
+    print('Started .0.', end='', flush=True)
+    outCount = 10
+    for (info, block) in reader:
+        if info.getPercent() > outCount:
+            print('.'+str(int(outCount))+'.', end='', flush=True)
+            outCount = outCount + 10
+                
+        blkShape = block.shape
+        blkBands = block.reshape((blkShape[0], (blkShape[1]*blkShape[2])))
+        
+        blkBandsTrans = numpy.transpose(blkBands)
+        
+        if noData is not None:
+            blkBandsTrans = blkBandsTrans[(blkBandsTrans!=noData).all(axis=1)]
+        
+        if blkBandsTrans.shape[0] > 0:
+            nSamp = int((blkBandsTrans.shape[0])/pxlNSample)
+            nSampRange = numpy.arange(0, nSamp, 1)*pxlNSample
+            blkBandsTransSamp = blkBandsTrans[nSampRange]
+            
+            if first:
+                outArr = blkBandsTransSamp
+                first = False
+            else:
+                outArr = numpy.concatenate((outArr, blkBandsTransSamp), axis=0)
+    print('. Completed')
+    return outArr
+
 
